# -*- coding: utf-8 -*-
# python program to automatically ramp quantiFlash intensity

# @author: Stephan Benke, Cytometry Facility, University Zurich
# @author: Konrad von Volkmann, APE GmbH, Berlin
# @author: Sjoerd Visser, APE GmbH, Berlin
# @author: Toralf Kaiser, DRFZ, Berlin
# @author: Sebastian Falkowski, APE GmbH, Berlin

# -------------------------------------
# Don't change or delete this section
# -------------------------------------
# @inistart

# [script]
# name = 1. Signal Ramping

# [param1]
# name = startInt
# label = Start Intensity
# value = -94
# mandatory = true
# datatype = int

# [param2]
# name = stopInt
# label = Stop Intensity
# value = 0
# mandatory = true
# datatype = int

# [param3]
# name = step
# label = Step Size
# value = 2
# mandatory = true
# datatype = int

# [param4]
# name = trigInt
# label = Trigger Intensity
# value = 50
# mandatory = true
# datatype = int

# [param5]
# name = nSamples
# label = Nb. of samples
# value = 10000
# mandatory = true
# datatype = int

# [param6]
# name = tcpPort
# label = TCP Port
# mandatory = true
# getter = MenuSettingsController.TextFieldTCPPort
# datatype = int

# [param7]
# name = countRate
# label = Count Rate
# value = 4000
# mandatory = true
# datatype = int

# @iniend

# -------------------------------------
# Preparations
# -------------------------------------
import os
import sys
import array
import struct
import time
import argparse
import numpy as np
import configparser
from win32com.shell import shell, shellcon

from ape.quantiflash import QuantiFlash
from ape.gui import Control

# -------------------------------------
# Script version
script_version = '1.0.130'
# -------------------------------------

gui = Control()

argparser = argparse.ArgumentParser(description='Description')
argparser.add_argument('--countRate', metavar='countRate', type=int)
argparser.add_argument('--tcpPort', metavar='tcpPort', type=int)
argparser.add_argument('--trigInt', metavar='trigInt', type=int)
argparser.add_argument('--nSamples', metavar='nSamples', type=int)
argparser.add_argument('--startInt', metavar='startInt', type=int)
argparser.add_argument('--stopInt', metavar='stopInt', type=int)
argparser.add_argument('--step', metavar='step', type=int)

args = argparser.parse_args()

# define local variable names
startInt = args.startInt
stopInt = args.stopInt
step = args.step
trigInt = args.trigInt
countRate = int(args.countRate)
tcpPort = args.tcpPort
nSamples = float(args.nSamples)
time_to_sleep = float(nSamples/countRate)

# output to the Python console
print("Start intensity: %d" % startInt)
print("Stop intensity: %d" % stopInt)
print("Intensity step: %d" % step)
print("Trigger intensity: %d" % trigInt)
print("Event rate: %d" % countRate)
print("Nb. of samples per intensity: %d" % nSamples)

# -------------------------------------
# Connection to the quantiflash software
# -------------------------------------

quantiflash = QuantiFlash()
print("IDN: "+quantiflash.idn())

# -------------------------------------
# Start data acquisition
# -------------------------------------
quantiflash.setTriggerIntensity(0)
quantiflash.setPulseRate(countRate)

# Communication with user to start the data acquisition of the cytometer
# print("Please start data recording on your cytometer ...")
# print("To continue press \"enter\"")

# wait for user confirmation
if gui.confirm("Please start data recording on your cytometer ...\n continue?") != 'OK':
	sys.exit()
timeStart = time.strftime("%X")
date1 = time.strftime("%Y%m%d")
date2 = time.strftime("%d.%m.%Y")

# -------------------------------------
# Measurement of electronic noise
# -------------------------------------
gui.task("Starting electronic noise measurement...")
quantiflash.setPulseIntensity("OFF")
quantiflash.setTriggerIntensity(trigInt)
print("Intensity : " + quantiflash.getPulseIntensity())
time.sleep(time_to_sleep)  # wait 0.1 ms
quantiflash.setTriggerIntensity(0)
# 0.2 second dark in between for better gating on time signal
time.sleep(0.2)
gui.task("Electronic noise measurement done.")


# -------------------------------------
# signal ramping, quantiflash intensities defined by input parameters
# -------------------------------------
intensities = np.arange(startInt, stopInt+step, step)
NN = intensities.shape[0]
print("number of intensities to measure: {:d}".format(NN))
counter = 1.0
print("Starting signal ramping...")
for tmp_int in intensities:
    quantiflash.setPulseIntensity(int(tmp_int))
    #time.sleep(0.1)
    quantiflash.setTriggerIntensity(trigInt)
    print("Intensity : " + quantiflash.getPulseIntensity())
    gui.task("Intensity : {:f}".format(tmp_int))
    time.sleep(time_to_sleep)  # wait 0.1 ms
    # 0.2 second dark in between for better gating on time signal
    quantiflash.setTriggerIntensity(0)
    time.sleep(0.2)
    gui.progress("{:f}".format(counter/NN*100))
    counter += 1


print("Signal ramping done.")
print("Measurement finished.")
print("Saving quantiFlash script settings.")

# -------------------------------------
# Create Config file in directory
# -------------------------------------
config = configparser.ConfigParser()
config['Main'] = {
    'Date1': date1,
    'Date2': date2,
    'Time': timeStart,
    'Device': quantiflash.idn(),
    'Port': tcpPort
}

config['quantiFlash'] = {
    'startInt': startInt,
    'stopInt': stopInt,
    'stepInt': step,
    'triggerInt': trigInt,
    'countRate': countRate,
    'nSamples': nSamples
}

config['Software'] = {
    'SignalRamping Software version': script_version,
    'Create_Consistency_file Software version': 'Undefined',
    'Eval_SNR_DNR': 'Undefined'
}

config['Evaluation'] = {
    'resolution': 'Undefined',
    'detectors': 'height',
    'v_ref': 550
}

config['Output'] = {
    'result': 'True',
    'raw_median': 'False',
    'raw_sd': 'True',
    'raw_mean': 'True',
    'k_factor': 'True'
}


# default path: 'USER DOCUMENTS'\quantiFlash\
cfg_path = os.path.join(shell.SHGetFolderPath(0, shellcon.CSIDL_PERSONAL, None, 0),'quantiFlash')
print(cfg_path)
if not os.path.exists(cfg_path):
    os.makedirs(cfg_path)
cfg_filename = 'quantiFlash_script_settings_%s.cfg' % time.strftime("%Y%m%d_%H%M%S")
cfg_file = os.path.join(cfg_path,cfg_filename)
with open(cfg_file, 'w') as f:
    config.write(f)

quantiflash.disconnect()
